/*
 *                 [[ Frozen Motion ]]
 *
 * Copyright (c) 2004 Glenn Sanson.
 *
 * This code is distributed under the GNU General Public License 
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * version 2, as published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 * Artwork:
 *    Amaury Amblard-Ladurantie <amaury at linuxfr.org>
 *      (the bubbles)
 *
 * Design & Programming:
 *    Glenn Sanson <glenn.sanson at free.fr>
 *      (whole Java sourcecode, including JIGA classes 
 *             http://glenn.sanson.free.fr/jiga/)
 *
 *          [[ http://glenn.sanson.free.fr/fmotion/home.html ]]
 */

import net.library.jiga.*;

import java.awt.*;

public class CursorSprite extends BubbleSprite {
	
	public final static double SPEED_UP = 0.5;
	
	int rotation;
	double angle;
	
	public CursorSprite(Rectangle rect, Image[] image, Game game) {
		
		super(rect, image, TYPE_CURSOR, game);

		rotation = 0;
		angle = (((double)rotation - 9) / 18.) * Math.PI;
	}
		
	// true = turn left / false = turn right
	public void rotate(boolean turnLeft) {
		
		// Select new picture
		if (turnLeft) {
			rotation--;
			
			if (rotation < 0) {
				rotation += 36;
			}
		}
		else
		{
			rotation++;
			
			if (rotation > 35) {
				rotation -= 36;
			}
		}
		
		angle = (((double)rotation - 9) / 18.) * Math.PI;
		
		this.forceRefresh();
	}
	
	public double getAngle() {
		return angle;
	}
	
	public void accelerate() {
		this.getSpeedVector().addToMax(new SpeedVector(SPEED_UP*Math.cos(angle), SPEED_UP*Math.sin(angle)), MAX_SPEED);		
		isMoving = true;
	}
	
	public void increaseTime() {}
	
	public final void paint(Graphics g, GameApplet applet) {
				
		int coordX = realPosition.x - 32 * (rotation % 6);
		int coordY = realPosition.y - 32 * (rotation / 6);
		
		g.setClip(realPosition);
		g.drawImage(bubbleImage[0], coordX, coordY, applet);

		if (horizontalExtend) {
			g.setClip(realPosition.x - applet.getWidth(), realPosition.y, 32, 32);
			g.drawImage(bubbleImage[0], coordX - applet.getWidth(), coordY, applet);
			
			if (verticalExtend) {
				g.setClip(realPosition.x - applet.getWidth(), realPosition.y - applet.getHeight(), 32, 32);
				g.drawImage(bubbleImage[0], coordX - applet.getWidth(), coordY - applet.getHeight(), applet);
			}
		}	
		
		if (verticalExtend) {
			g.setClip(realPosition.x, realPosition.y - applet.getHeight(), 32, 32);
			g.drawImage(bubbleImage[0], coordX, coordY - applet.getHeight(), applet);			
		}
	}	
}