/*
 *                 [[ Frozen Motion ]]
 *
 * Copyright (c) 2004 Glenn Sanson.
 *
 * This code is distributed under the GNU General Public License 
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * version 2, as published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 * Artwork:
 *    Amaury Amblard-Ladurantie <amaury at linuxfr.org>
 *      (the bubbles)
 *
 * Design & Programming:
 *    Glenn Sanson <glenn.sanson at free.fr>
 *      (whole Java sourcecode, including JIGA classes 
 *             http://glenn.sanson.free.fr/jiga/)
 *
 *          [[ http://glenn.sanson.free.fr/fmotion/home.html ]]
 */

import net.library.jiga.*;

import java.awt.*;

public class Elastic extends Sprite {
	
	Rectangle screen;
	BubbleSprite bs1, bs2;
	Point lastKnown1, lastKnown2;
	
	double initialLength;
	
	public Elastic(BubbleSprite bs1, BubbleSprite bs2, Rectangle screen) {
		
		super(new Rectangle());
		
		this.bs1 = bs1;
		this.bs2 = bs2;
		this.screen = screen;
		this.initialLength = Math.sqrt(Math.pow(bs1.realX-bs2.realX, 2.)+Math.pow(bs1.realY-bs2.realY, 2.));
		
		calculateArea();
	}
	
	void calculateArea() {
		int posX = Math.min((bs1.getRealPosition().x+bs1.getRealPosition().width/2) % screen.width, (bs2.getRealPosition().x+bs2.getRealPosition().width/2) % screen.width);
		int posY = Math.min((bs1.getRealPosition().y+bs1.getRealPosition().height/2) % screen.height, (bs2.getRealPosition().y+bs2.getRealPosition().height/2) % screen.height);
		
		int width = Math.abs(((bs1.getRealPosition().x+bs1.getRealPosition().width/2) % screen.width) - ((bs2.getRealPosition().x+bs2.getRealPosition().width/2) % screen.width));
		int height = Math.abs(((bs1.getRealPosition().y+bs1.getRealPosition().height/2) % screen.height) - ((bs2.getRealPosition().y+bs2.getRealPosition().height/2) % screen.height));
		
		this.changeSpriteArea(new Rectangle(posX-1, posY-1, width+2, height+2));
		
		lastKnown1 = bs1.getRealPosition().getLocation();
		lastKnown2 = bs2.getRealPosition().getLocation();
	}
	
	// false = remove
	boolean check(Game game) {
		if (bs1.getType() == BubbleSprite.TYPE_REMOVED || bs2.getType() == BubbleSprite.TYPE_REMOVED) {
			game.eliminateSprite(this);
			
			return false;
		}
		
		if (!lastKnown1.equals(bs1.getRealPosition().getLocation()) || !lastKnown2.equals(bs2.getRealPosition().getLocation())) {
			this.forceRefresh();
			calculateArea();
		}
		
		changeSpeed();
		
		return true;
	}
	
	void changeSpeed() {
		double deltaX = (bs1.getRealPosition().x+bs1.getRealPosition().width/2) % screen.width - (bs2.getRealPosition().x+bs2.getRealPosition().width/2) % screen.width;
		double deltaY = (bs1.getRealPosition().y+bs1.getRealPosition().height/2) % screen.height - (bs2.getRealPosition().y+bs2.getRealPosition().height/2) % screen.height;
		
		double currentLength = Math.sqrt(deltaX*deltaX + deltaY * deltaY);
		double newSpeed = initialLength/currentLength;
		
		if (newSpeed < 1.) {
			newSpeed = -1./newSpeed;
		} 
						
		double normalize = 0.2 * newSpeed / currentLength;

		bs1.setSpeedVector(bs1.getSpeedVector().addToMax(new SpeedVector(deltaX*normalize, deltaY*normalize), 5.));
		bs2.setSpeedVector(bs2.getSpeedVector().addToMax(new SpeedVector(-deltaX*normalize, -deltaY*normalize), 5.));		
	}
	
	public void paint(Graphics g, GameApplet applet) {
		g.setClip(this.getSpriteArea());

		g.setColor(Color.white);

		g.drawLine((bs1.getRealPosition().x+bs1.getRealPosition().width/2) % screen.width,
				   (bs1.getRealPosition().y+bs1.getRealPosition().height/2) % screen.height,
				   (bs2.getRealPosition().x+bs2.getRealPosition().width/2) % screen.width,
				   (bs2.getRealPosition().y+bs2.getRealPosition().height/2) % screen.height);
	}	
}