/*
 *                 [[ Frozen Motion ]]
 *
 * Copyright (c) 2004 Glenn Sanson.
 *
 * This code is distributed under the GNU General Public License 
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * version 2, as published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 * Artwork:
 *    Amaury Amblard-Ladurantie <amaury at linuxfr.org>
 *      (the bubbles)
 *
 * Design & Programming:
 *    Glenn Sanson <glenn.sanson at free.fr>
 *      (whole Java sourcecode, including JIGA classes 
 *             http://glenn.sanson.free.fr/jiga/)
 *
 *          [[ http://glenn.sanson.free.fr/fmotion/home.html ]]
 */

import net.library.jiga.*;
import java.awt.*;
import java.util.Vector;

import java.util.Random;

public class Game extends GameScreen {

	public final static int CURSOR_TURN = 0;
	public final static int CURSOR_SPEEDUP = 1;
		
	public final static int KEY_UP = 38;
	public final static int KEY_LEFT = 37;
	public final static int KEY_RIGHT = 39;
	public final static int KEY_SHIFT = 16;
	public final static int KEY_ESCAPE = 27;
	
	public final static int GAME_RUNNING = 0;
	public final static int GAME_WON = 1;
	public final static int GAME_LOST = 2;
	
	public final static String GAME_LOST_TXT = "too slow!!!";
	public final static String GAME_LOST_TXT2 = "try again?";
	public final static String GAME_WON_TXT = "press fire";
	public final static String GAME_WON_TXT2 = "to continue";
	
	int gameState;
	
    Image background;
	
	Vector movingBubbles;
	Vector staticBubbles;
	Vector elastics;
	Vector explosions;
	Vector popImages;
	
	Random randExplosion; // used for explosions
	
	CursorSprite player;
	Image[] playerImage;
	
	Image[][] bubbles;
	Image[][] mini;
	Image[] walls;
	Image popImage;
	Image lifeImage;
	
	byte[][] wallsModel;
	
	LifeManager lifeManager;
	LevelManager levelManager;
	
	BigFont bigFont;
	SmallFont smallFont;
	Vector textVector;
	
	SoundManager soundManager;
	
	// Level parameters
	int[] parameters;
	
	// Init sequence
	boolean released, pressed;
	
    public Game(GameApplet gameApplet) {
        super(gameApplet);
		
		GameMedia media = gameApplet.getGameMedia();
		background = media.loadImage("background.gif");	
		soundManager = (SoundManager)gameApplet.getGameContext().getObject("soundManager");
		
		// Initialize bubbles
		movingBubbles = new Vector();
		staticBubbles = new Vector();
		
		// Initialize elastics
		elastics = new Vector();
		
		// Initialize explosions
		explosions = new Vector();
		randExplosion = new Random(System.currentTimeMillis());
		
		// initialize pop
		popImages = new Vector();
		
		// Get pictures
		bubbles = new Image[3][2];
		bubbles[0][0] = media.loadImage("bubble-1.gif");
		bubbles[1][0] = media.loadImage("bubble-2.gif");
		bubbles[2][0] = media.loadImage("bubble-3.gif");
		bubbles[0][1] = media.loadImage("bubble-4.gif");
		bubbles[1][1] = media.loadImage("bubble-5.gif");
		bubbles[2][1] = media.loadImage("bubble-6.gif");
		
		// get mini pictures
		mini = new Image[3][2];
		mini[0][0] = media.loadImage("mini-1.gif");
		mini[1][0] = media.loadImage("mini-2.gif");
		mini[2][0] = media.loadImage("mini-3.gif");
		mini[0][1] = media.loadImage("mini-4.gif");
		mini[1][1] = media.loadImage("mini-5.gif");
		mini[2][1] = media.loadImage("mini-6.gif");

		// Get Walls
		walls = new Image[5];
		walls[0] = media.loadImage("block-1.gif");
		walls[1] = media.loadImage("block-2.gif");
		walls[2] = media.loadImage("block-3.gif");
		walls[3] = media.loadImage("block-4.gif");
		walls[4] = media.loadImage("block-5.gif");
		
		// Get Pop image
		popImage = media.loadImage("pop.gif");
				
		// CurrentLife 
		lifeImage = media.loadImage("life.gif");
		
		// LevelManager
		levelManager = (LevelManager)gameApplet.getGameContext().getObject("levelManager");
		Level currentLevel = levelManager.getLevel();		
		
		// Font
		bigFont = (BigFont)gameApplet.getGameContext().getObject("bigFont");
		smallFont = (SmallFont)gameApplet.getGameContext().getObject("smallFont");
		
		// Walls
		wallsModel = currentLevel.getWalls();

		// Init bubbles
		Vector bubblesVector = currentLevel.getBubbles();
		parameters = currentLevel.getParameters();
		
		// Cursor
		playerImage = new Image[1];
		playerImage[0] = media.loadImage("arrow.gif");
		player = new CursorSprite(new Rectangle(((int[])bubblesVector.elementAt(0))[1],((int[])bubblesVector.elementAt(0))[2], 32, 32), playerImage, this);
		this.addStaticSprite(player);		
		
		// Other bubbles
		for (int i=1 ; i<bubblesVector.size() ; i++) {
			this.addStaticSprite(new BubbleSprite(new Rectangle(((int[])bubblesVector.elementAt(i))[1], ((int[])bubblesVector.elementAt(i))[2], 32, 32), bubbles[((int[])bubblesVector.elementAt(i))[0]], BubbleSprite.TYPE_NORMAL,this));
		}
		
		// Init Links
		Vector linksVector = currentLevel.getLinks();
		
		for (int i=0 ; i<linksVector.size() ; i++) {
			Elastic e = new Elastic((BubbleSprite)staticBubbles.elementAt(((int[])linksVector.elementAt(i))[0]), 
									(BubbleSprite)staticBubbles.elementAt(((int[])linksVector.elementAt(i))[1]), 
									new Rectangle(0, 0, this.getGameApplet().getWidth(), this.getGameApplet().getHeight()));
			elastics.addElement(e);
			this.spriteToBack(e);
		}
				
		// Life
		lifeManager = new LifeManager(lifeImage, currentLevel.getParameters());
		this.addSprite(lifeManager);
		
		// Init Description
		textVector = initText(currentLevel, bigFont, smallFont);
		
		for (int i=0 ; i<textVector.size() ; i++) {
			this.addSprite((Sprite)textVector.elementAt(i));
		}
	}	
	
    public void initBackground() {
		addToBackground(background, new Point(0, 0));
		
		// Create walls
		for (int i=0 ; i<16 ; i++) {
			for (int j=0 ; j<12 ; j++) {
				
				int currentColor = wallsModel[i][j];
				
				if (currentColor != 0) {
					
					currentColor--;
					byte[] forms = getNeighbors(i, j);
					
					addToBackground(walls[currentColor], new Point(i*40-forms[0], j*40), new Rectangle(i*40, j*40, 20, 20));
					addToBackground(walls[currentColor], new Point(i*40+20-forms[1], j*40-20), new Rectangle(i*40+20, j*40, 20, 20));
					addToBackground(walls[currentColor], new Point(i*40-forms[2], j*40-20), new Rectangle(i*40, j*40+20, 20, 20));
					addToBackground(walls[currentColor], new Point(i*40+20-forms[3], j*40-40), new Rectangle(i*40+20, j*40+20, 20, 20));
				}
			}
		}
    }
	
	// Create sprite of the initial text
	public Vector initText(Level currentLevel, BigFont bigFont, SmallFont smallFont) {
		Vector textVector = new Vector();
		
		int size = this.getTextSize(currentLevel.getName(), bigFont);
		TextSprite ts = new TextSprite(new Rectangle((640 - size) >> 1, 179, size, 30), bigFont, currentLevel.getName());
		textVector.addElement(ts);

		size = this.getTextSize("difficulty", smallFont);
		ts = new TextSprite(new Rectangle((640 - size) >> 1, 218, size, 30), smallFont, "difficulty");
		textVector.addElement(ts);			
		
		String stars = new String();
		for (int i=0 ; i<currentLevel.getDifficulty() ; i++) {
			stars += "";
		}
		size = this.getTextSize(stars, bigFont);
		ts = new TextSprite(new Rectangle((640 - size) >> 1, 224, size, 30), bigFont, stars);
		textVector.addElement(ts);	

		size = this.getTextSize("designed by", smallFont);
		ts = new TextSprite(new Rectangle((640 - size) >> 1, 264, size, 30), smallFont, "designed by");
		textVector.addElement(ts);			

		size = this.getTextSize(currentLevel.getAuthor(), bigFont);
		ts = new TextSprite(new Rectangle((640 - size) >> 1, 274, size, 30), bigFont, currentLevel.getAuthor());
		textVector.addElement(ts);	
		
		return textVector;
	}
	
	public int getTextSize(String text, GameFont font) {
		int size = 0;
		
		for (int i=0 ; i<text.length() ; i++) {
			size += font.charSize(text.charAt(i));
			size += font.SEPARATOR_WIDTH;
		}
		
		return size - font.SEPARATOR_WIDTH;
	}
	
	public int[] getParameters() {
		return parameters;
	}	
	
	public int getWallValue(int x, int y) {
		return wallsModel[x][y];
	}
		
	byte[] getNeighbors(int x, int y) {
		
		// Return right picture in the block for each corner (UL, UR, DL, DR)
		byte[] output = new byte[4];
		for (int i=0 ; i<4 ; i++) {
			output[i] = 40;
		}
		
		int currentColor = wallsModel[x][y];
		
		int minX = x-1;
		if (minX == -1) {
			minX = 15;
		}
		
		int maxX = x+1;
		if (maxX == 16) {
			maxX = 0;
		}
		
		int minY = y-1;
		if (minY == -1) {
			minY = 11;
		}
		
		int maxY = y+1;
		if (maxY == 12) {
			maxY = 0;
		}
				
		// Horizontal
		if (wallsModel[minX][y] == currentColor) {
			output[0] -= 40;
			output[2] -= 40;
		}
		if (wallsModel[maxX][y] == currentColor) {
			output[1] -= 40;
			output[3] -= 40;
		}
		
		// Vertical
		if (wallsModel[x][minY] == currentColor) {
			output[0] += 60;
			output[1] += 60;
		}
		if (wallsModel[x][maxY] == currentColor) {
			output[2] += 60;
			output[3] += 60;
		}
		
		// Diagonale
		if (wallsModel[minX][minY] == currentColor) {
			if (output[0] != 40) {
				output[0] += 20;
			}
		}
		if (wallsModel[maxX][minY] == currentColor) {
			if (output[1] != 40) {
				output[1] += 20;
			}
		}
		if (wallsModel[minX][maxY] == currentColor) {
			if (output[2] != 40) {
				output[2] += 20;
			}
		}
		if (wallsModel[maxX][maxY] == currentColor) {
			if (output[3] != 40) {
				output[3] += 20;
			}
		}		
		
		return output;
	}
	
	public void addStaticSprite(BubbleSprite bs) {
		staticBubbles.addElement(bs);
		addSprite(bs);
	}
	
	public void eliminateSprite(BubbleSprite bs) {
		this.removeSprite(bs);
		staticBubbles.removeElement(bs);
		movingBubbles.removeElement(bs);
		
		switch (bs.getType())
		{
			case BubbleSprite.TYPE_MINI :
				lifeManager.increaseLife();
			break;
			case BubbleSprite.TYPE_COLLISION :
				PopImage pi = new PopImage(bs.getRealPosition(), popImage);
				this.addSprite(pi);
				popImages.addElement(pi);
				soundManager.playSound(FrozenMotion.SOUND_POP);
			break;
			case BubbleSprite.TYPE_NORMAL :
				lifeManager.decreaseLife();
			break;
		}
				
		bs.remove();
	}
	
	public void eliminateSprite(Elastic el) {
		this.removeSprite(el);
		elastics.removeElement(el);
	}
	
	public void eliminateSprite(Explosion ex) {
		this.removeSprite(ex);
		explosions.removeElement(ex);
	}
	
	public void eliminateSprite(PopImage pi) {
		this.removeSprite(pi);
		popImages.removeElement(pi);
	}	
	
	public void addExplosion(Rectangle r, Image img) {
		Rectangle miniRect = new Rectangle(r.x+r.width/2-8, r.y+r.height/2-8, 16, 16);
		
		for (int i=0 ; i<5 ; i++) {
			Explosion explosion = new Explosion(miniRect, img, randExplosion);
			
			this.addSprite(explosion);
			explosions.addElement(explosion);
		}
		
		soundManager.playSound(FrozenMotion.SOUND_EXPLOSION);
	}
	
	public Image getMini(Image img) {
		for (int i=0 ; i<bubbles.length ; i++) {
			if (img == bubbles[i][0]) {
				return mini[i][0];
			}
		}
		
		// When not found
		return mini[0][0];
	}
	
	public Image[] getMini(BubbleSprite bs1, BubbleSprite bs2) {
		
		if (bs1.getImage() == bubbles[0][0]) {
			if (bs2.getImage() == bubbles[1][0]) {
				return mini[2];
			} else {
				return mini[1];
			}
		} else if (bs1.getImage() == bubbles[1][0]) {
			if (bs2.getImage() == bubbles[0][0]) {
				return mini[2];
			} else {
				return mini[0];
			}			
		} else {
			if (bs2.getImage() == bubbles[0][0]) {
				return mini[1];
			} else {
				return mini[0];
			}			
		}
	}
	
	public Image[] getFull(BubbleSprite bs) {
		for (int i=0 ; i<3 ; i++) {
			if (mini[i][0] == bs.getImage()) {
				return bubbles[i];
			}
		}
		
		return null;
	}
		
	// M=Mobile / S=Static	
    public void play(int[] keyCodes, Point mouseCoord, boolean leftButton, boolean rightButton) {
		
		// Check key pressed
		int[] move = new int[2];
		
		if (!released) {
			if (keyCodes.length == 0) {
				released = true;
			} 
			else if (gameState == this.GAME_RUNNING) {
				return;
			}
		}
		else if (!pressed) {
			if (keyCodes.length == 0) {
				return;
			}
			else {
				pressed = true;
				for (int i=0 ; i<textVector.size() ; i++) {
					this.removeSprite((Sprite)textVector.elementAt(i));
				}
			}
		}
		
		for (int i=0 ; i<keyCodes.length ; i++) {
			
			int current = keyCodes[i];
						
			if (current == KEY_ESCAPE) {
				this.getGameApplet().setCurrentScreen(new LevelSelector(this.getGameApplet()));
			}
			
			if (current == KEY_LEFT || current == KEY_RIGHT) {				
				if (move[CURSOR_TURN] == 0)	{
					move[CURSOR_TURN] = current;
				}
				else {
					move[CURSOR_TURN] = 0;
				}
			}
			
			if (current == KEY_UP || current == KEY_SHIFT) {
				move[CURSOR_SPEEDUP] = KEY_UP;
			}
		}
		
		
		// Move Vector (player)
		if (move[CURSOR_TURN] == this.KEY_LEFT) {
			player.rotate(true);
		}
		else if (move[CURSOR_TURN] == this.KEY_RIGHT) {
			player.rotate(false);
		}
		
		if (move[CURSOR_SPEEDUP] == KEY_UP)
		{
			if (!player.isMoving())	{
				staticBubbles.removeElement(player);
				movingBubbles.addElement(player);
			}
			
			if (released) {
				switch(gameState) {
					case GAME_RUNNING :
						player.accelerate();
					break;
					case GAME_LOST :
						this.getGameApplet().setCurrentScreen(new Game(this.getGameApplet()));					
					break;
					case GAME_WON :
						if (levelManager.nextUnchallengedLevel()) {
							this.getGameApplet().setCurrentScreen(new Game(this.getGameApplet()));
						}
						else {
							this.getGameApplet().setCurrentScreen(new HighscoresScreen(this.getGameApplet()));
						}
					break;
				}
			}
		}
		
		// Move bubbles
		for (int i=0 ; i<movingBubbles.size() ; i++) {
			if (((BubbleSprite)movingBubbles.elementAt(i)).moveBubble()) {
				soundManager.playSound(FrozenMotion.SOUND_COLLISION);
			}
		}

		// Collision S<->M
		for (int i=0 ; i<movingBubbles.size() ; i++) {
			BubbleSprite current = (BubbleSprite)movingBubbles.elementAt(i);
			
			for (int j=0 ; j<staticBubbles.size() ; j++) {
				if (current.checkCollision((BubbleSprite)staticBubbles.elementAt(j))) {
					if (current.getType() != BubbleSprite.TYPE_REMOVED) {
						soundManager.playSound(FrozenMotion.SOUND_BUBBLES);
					}
				}
			}
		}		

		// Collision M<->M
		for (int i=0 ; i<movingBubbles.size()-1 ; i++) {
			BubbleSprite current = (BubbleSprite)movingBubbles.elementAt(i);
			
			for (int j=i+1 ; j<movingBubbles.size() ; j++) {
				if (current.checkCollision((BubbleSprite)movingBubbles.elementAt(j))) {
					if (current.getType() != BubbleSprite.TYPE_REMOVED) {
						soundManager.playSound(FrozenMotion.SOUND_BUBBLES);
					}
				}
			}
		}
	
		// Integrate new moving bubbles
		for (int i=0 ; i<staticBubbles.size() ; i++) {
			BubbleSprite current = (BubbleSprite)staticBubbles.elementAt(i);
			
			if (current.isMoving()) {
				staticBubbles.removeElement(current);
				movingBubbles.addElement(current);
			}
		}		
		
		// Reduce speed
		for (int i=0 ; i<movingBubbles.size() ; i++) {
			BubbleSprite current = (BubbleSprite)movingBubbles.elementAt(i);
			
			if (current.reduceSpeed()) {
				movingBubbles.removeElement(current);
				staticBubbles.addElement(current);
			}
		}

		// Increase time
		// Reset Collision detection
		for (int i=0 ; i<staticBubbles.size() ; i++) {
			((BubbleSprite)staticBubbles.elementAt(i)).increaseTime();
		}
		for (int i=0 ; i<movingBubbles.size() ; i++) {
			((BubbleSprite)movingBubbles.elementAt(i)).increaseTime();
		}
		
		// Move Elastics
		for (int i=0 ; i<elastics.size() ; i++) {
			((Elastic)elastics.elementAt(i)).check(this);
		}
		
		// Calculate explosions
		for (int i=0 ; i<explosions.size() ; i++) {
			((Explosion)explosions.elementAt(i)).move(this);
		}
		
		// Check pop
		for (int i=0 ; i<popImages.size() ; i++) {
			((PopImage)popImages.elementAt(i)).increaseTime(this);
		}
				
		if (gameState != this.GAME_RUNNING) {
			return;
		}
				
		// check if game is lost
		if (lifeManager.gameLost()) {
			this.eliminateSprite(player);
			for (int i=0 ; i<3 ; i++) {
				this.addExplosion(player.getRealPosition(), mini[i][0]);
				this.addExplosion(player.getRealPosition(), mini[i][1]);
			}
			
			gameState = this.GAME_LOST;
			
			this.addSprite(new TextSprite(new Rectangle((640-this.getTextSize(GAME_LOST_TXT, bigFont)) >> 1, 210, this.getTextSize(GAME_LOST_TXT, bigFont), 30), bigFont, GAME_LOST_TXT));
			this.addSprite(new TextSprite(new Rectangle((640-this.getTextSize(GAME_LOST_TXT2, bigFont)) >> 1, 243, this.getTextSize(GAME_LOST_TXT2, bigFont), 30), bigFont, GAME_LOST_TXT2));
			
			released = false;
			
			return;
		}
		
		// Victory == count(bubbles) = 1 (never forget the cursor)
		if (staticBubbles.size() + movingBubbles.size() == 1) {
			levelManager.getLevel().setFinished();
			
			gameState = this.GAME_WON;

			this.addSprite(new TextSprite(new Rectangle((640-this.getTextSize(GAME_WON_TXT, bigFont)) >> 1, 210, this.getTextSize(GAME_WON_TXT, bigFont), 30), bigFont, GAME_WON_TXT));
			this.addSprite(new TextSprite(new Rectangle((640-this.getTextSize(GAME_WON_TXT2, bigFont)) >> 1, 243, this.getTextSize(GAME_WON_TXT2, bigFont), 30), bigFont, GAME_WON_TXT2));
			
			released = false;
			
			return;
		}
		
		// Increase time
		lifeManager.increaseTime();		
    }
	
}