/*
 *                 [[ Frozen Motion ]]
 *
 * Copyright (c) 2004 Glenn Sanson.
 *
 * This code is distributed under the GNU General Public License 
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * version 2, as published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 * Artwork:
 *    Amaury Amblard-Ladurantie <amaury at linuxfr.org>
 *      (the bubbles)
 *
 * Design & Programming:
 *    Glenn Sanson <glenn.sanson at free.fr>
 *      (whole Java sourcecode, including JIGA classes 
 *             http://glenn.sanson.free.fr/jiga/)
 *
 *          [[ http://glenn.sanson.free.fr/fmotion/home.html ]]
 */

import net.library.jiga.*;
import java.awt.*;
import java.awt.event.*;
import java.net.*;

public class HighscoresScreen extends GameScreen {

	private final static String PARAMETER_DATE = "date";
	private final static String PARAMETER_NAME = "player";
	private final static String PARAMETER_SCORE = "score";
	private final static String PARAMETER_SET_ID = "id";
	
	private final static String[] OFFLINE_RANKING = {"read the rules", "no comment", "time to wake up", 
													 "try frozen bubble", "too hard?", "tired?", "stop coffee!", 
													 "could do better", "not too bad", "well done", 
													 "hard gamer", "keybord master", "genius", "a.i.", 
													 "did you write this game?", "killer", "serial winner", 
													 "incredible", "unbelivable", "perfect"};
	
	private final static String HIGHSCORES = "highscores";
	private final static String OFFLINE_FINAL_SCORE = "final score";
	private final static String OFFLINE_COMMENT = "comment";
	private final static String ONLINE_PROMPT = "enter your name";
	
	BigFont bigFont;
	SmallFont smallFont;	

	LevelManager levelManager;
	
	boolean activate;
	private int lastKeyPressed;
	
	boolean onlineHighscores;
	long timer;
	
    Image background;
	
	TextSprite playerName;
	
    public HighscoresScreen(GameApplet gameApplet) {
		super(gameApplet);
		
		GameMedia media = gameApplet.getGameMedia();
		background = media.loadImage("background.gif");			
		
		// Font
		bigFont = (BigFont)gameApplet.getGameContext().getObject("bigFont");
		smallFont = (SmallFont)gameApplet.getGameContext().getObject("smallFont");		
		
		// Level Manager
		levelManager = (LevelManager)gameApplet.getGameContext().getObject("levelManager");
		
		String dateString = gameApplet.getParameter(PARAMETER_DATE);
		if (dateString == null) {
			timer = -1;
			onlineHighscores = false;
		}
		else {
			timer = -1;
			onlineHighscores = false;
			
			try {
				// Convert date(sec) -> timer(pseudo-milli)
				
				timer = Long.parseLong(dateString);
				timer *= 1000;
				timer += (System.currentTimeMillis() % 1000);
				
				onlineHighscores = true;
			}
			catch(Exception e) {}
		}

		// Highscores title
		addSprite(new TextSprite(new Rectangle((gameApplet.getWidth() - this.getTextSize(this.HIGHSCORES, bigFont)) >> 1, 4, this.getTextSize(this.HIGHSCORES, bigFont), 30), bigFont, this.HIGHSCORES));
		
		if (onlineHighscores) {
			addSprite(new TextSprite(new Rectangle((gameApplet.getWidth() - this.getTextSize(this.ONLINE_PROMPT, smallFont)) >> 1, 215, this.getTextSize(this.ONLINE_PROMPT, smallFont), 8), smallFont, this.ONLINE_PROMPT));
			
			playerName = new TextSprite(new Rectangle(320, 225, 1, 30), bigFont, "");
			addSprite(playerName);
		}
		else { // Offline scores
			int nbStars = 1 + (int)(Math.floor(4.*levelManager.getScore()/levelManager.size()));
			String stars = new String();
			for (int i=0 ; i<nbStars ; i++) {
				stars += "";
			}
			
			addSprite(new TextSprite(new Rectangle((gameApplet.getWidth() - this.getTextSize(this.OFFLINE_FINAL_SCORE, smallFont)) >> 1, 180, this.getTextSize(this.OFFLINE_FINAL_SCORE, smallFont), 8), smallFont, this.OFFLINE_FINAL_SCORE));
			addSprite(new TextSprite(new Rectangle((gameApplet.getWidth() - this.getTextSize(stars, bigFont)) >> 1, 190, this.getTextSize(stars, bigFont), 30), bigFont, stars));
			
			int comment = (int)(Math.floor(19.*levelManager.getScore()/levelManager.size()));

			addSprite(new TextSprite(new Rectangle((gameApplet.getWidth() - this.getTextSize(this.OFFLINE_COMMENT, smallFont)) >> 1, 260, this.getTextSize(this.OFFLINE_COMMENT, smallFont), 8), smallFont, this.OFFLINE_COMMENT));
			addSprite(new TextSprite(new Rectangle((gameApplet.getWidth() - this.getTextSize(this.OFFLINE_RANKING[comment], bigFont)) >> 1, 270, this.getTextSize(this.OFFLINE_RANKING[comment], bigFont), 30), bigFont, this.OFFLINE_RANKING[comment]));			
		}
	}
	
	public int getTextSize(String text, GameFont font) {
		int size = 0;
		
		for (int i=0 ; i<text.length() ; i++) {
			size += font.charSize(text.charAt(i));
			size += font.SEPARATOR_WIDTH;
		}
		
		return size - font.SEPARATOR_WIDTH;
	}	
	
    public void initBackground() {
		addToBackground(background, new Point(0, 0));
	}	
	
	private boolean isValidChar(int code)
	{
		if (code>=KeyEvent.VK_0 && code<=KeyEvent.VK_9)
		{
			return true;
		}

		if (code>=KeyEvent.VK_A && code<=KeyEvent.VK_Z)
		{
			return true;
		}

		if (code == KeyEvent.VK_SPACE)
		{
			return true;
		}
		
		return false;
	}
	
    public void play(int[] keyCodes, Point mouseCoord, boolean leftButton, boolean rightButton) {
		
		// Anim management
		
		// Key management
		if (keyCodes.length == 0) {
			activate = true;
			lastKeyPressed = 0;
			return;
		}
		
		if (!activate) {
			return;
		}
		
		if (keyCodes[keyCodes.length-1] == lastKeyPressed) {
			return;
		}
		
		lastKeyPressed = keyCodes[keyCodes.length-1];
		
		if (!onlineHighscores || lastKeyPressed == KeyEvent.VK_ESCAPE) {
			levelManager.reset();
			this.getGameApplet().setCurrentScreen(new SplashScreen(this.getGameApplet()));
		}
		else if (lastKeyPressed == KeyEvent.VK_ENTER) {
			if (playerName.getText().length() == 0) {
				levelManager.reset();
				this.getGameApplet().setCurrentScreen(new SplashScreen(this.getGameApplet()));				
			}
			else {
				sendRecord();
			}
		}
		else if (lastKeyPressed == KeyEvent.VK_BACK_SPACE || lastKeyPressed == KeyEvent.VK_DELETE) {
			if (playerName.getText().length() > 0) {
				playerName.removeLastChar();
				
				int size = getTextSize(playerName.getText(), bigFont);
				playerName.changeSpriteArea(new Rectangle((this.getGameApplet().getWidth() - size) >> 1, 225, size, 30));
			}
		}
		else if (isValidChar(lastKeyPressed) && playerName.getText().length() < 16) {
			int size = getTextSize(playerName.getText()+String.valueOf((char)lastKeyPressed).toLowerCase(), bigFont);
			playerName.changeSpriteArea(new Rectangle((this.getGameApplet().getWidth() - size) >> 1, 225, size, 30));
			
			playerName.addCharacter((char)lastKeyPressed);
		}		
	}
	
	public void sendRecord()
	{
		String stringToSend = "update.php?";
		
		// Add level set ID
		stringToSend += this.PARAMETER_SET_ID + "=" + Integer.toString(levelManager.getDBNumber()) + "&";

		// Add time stamp
		stringToSend += this.PARAMETER_DATE + "=" + Long.toString(timer) + "&";
		
		// Add player name
		stringToSend += this.PARAMETER_NAME + "=" + convertToHTML(playerName.getText()) + "&";

		// Add player score
		stringToSend += this.PARAMETER_SCORE + "=" + Integer.toString(levelManager.getScore());
		
		try
		{
			this.getGameApplet().getAppletContext().showDocument(new URL(this.getGameApplet().getCodeBase(), stringToSend), "_self");
		}
		catch(Exception e)
		{
			System.err.println("Unable to send highscore");
		}
	}
	
	private String convertToHTML(String input)
	{
		String output = new String();
		
		int begin = 0;
		int end = input.indexOf(" ");
		
		while (end != -1)
		{
			output += input.substring(begin, end);
			output += "%20";
			
			begin = end + 1;
			
			end = input.indexOf(" ", begin);
		}	
		
		output += input.substring(begin);
		
		return output;
	}	
}