/*
 *                 [[ Frozen Motion ]]
 *
 * Copyright (c) 2004 Glenn Sanson.
 *
 * This code is distributed under the GNU General Public License 
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * version 2, as published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 * Artwork:
 *    Amaury Amblard-Ladurantie <amaury at linuxfr.org>
 *      (the bubbles)
 *
 * Design & Programming:
 *    Glenn Sanson <glenn.sanson at free.fr>
 *      (whole Java sourcecode, including JIGA classes 
 *             http://glenn.sanson.free.fr/jiga/)
 *
 *          [[ http://glenn.sanson.free.fr/fmotion/home.html ]]
 */

import net.library.jiga.*;
import java.util.Vector;

public class LevelManager {
	
	private final static String BEGIN_TAG = "[N:";
	
	private final static String SET_NAME_PARAMETER = "set_name";	
	private final static String SET_NAME_ORIGINAL = "default set";
	private final static String SET_NAME_DEFAULT = "frozen motion";
	
	private final static String SET_INFO_PARAMETER = "set_info";
	private final static String SET_INFO_ORIGINAL = "this is the default set of levels embedded in the frozen motion applet... have fun!!!";
	private final static String SET_INFO_DEFAULT = "this set was made by... well... i don t know !? this information is missing...";
	
	private final static String DB_NUMBER = "id";
	private final static int ORIGINAL_DB_NUMBER = 0;
	
	Vector list;
	int currentIndex;
	int dbNumber;
	
	private String setName, setInfo;
	
	public LevelManager(GameApplet applet) {
		
		list = new Vector();

		dbNumber = this.ORIGINAL_DB_NUMBER;
		
		if (applet.getParameter(DB_NUMBER) != null) {
			try {
				dbNumber = Integer.parseInt(applet.getParameter(DB_NUMBER));
			}
			catch(Exception e) {}
		}
		
		int levelIndex = 0;
		
		if (dbNumber != this.ORIGINAL_DB_NUMBER) {				
			String levelInfo = applet.getParameter("level"+Integer.toString(levelIndex));
		
			while (levelInfo != null) {
				list.addElement(new Level(Base64.fromBase64(levelInfo, Base64.BASE64_SAFE_ALPHABET)));
				
				levelIndex++;
				levelInfo = applet.getParameter("level"+Integer.toString(levelIndex));
			}
		}
			
		if (levelIndex == 0) { 
			// Use default levels
			String allLevels = null;
			
			try {
				allLevels = new String(applet.getGameMedia().loadData("levels.txt"), "ISO-8859-1");
			} 
			catch(Exception e) {
				System.err.println("Unrecognized encoding [ISO-8859-1]");
				allLevels = new String(); // Set with 0 levels
			}
			
			int begin = allLevels.indexOf(BEGIN_TAG);
		
			Level level = null;
		
			while (begin != -1) {
				int end = allLevels.indexOf(BEGIN_TAG, begin+BEGIN_TAG.length());
				
				if (end == -1) {
					level = new Level(allLevels.substring(begin));
				} else {
					level = new Level(allLevels.substring(begin, end));
				}
				
				if (level.isValid()) {
					list.addElement(level);
				}
				
				begin = end;
			}	
			
			setName = this.SET_NAME_ORIGINAL;
			setInfo = this.SET_INFO_ORIGINAL;
		}
		else {
			// User defined levels
			setName = applet.getParameter(this.SET_NAME_PARAMETER);
			if (setName == null) {
				setName = this.SET_NAME_DEFAULT;
			}

			setInfo = applet.getParameter(this.SET_INFO_PARAMETER);
			if (setInfo == null) {
				setInfo = this.SET_INFO_DEFAULT;
			}			
		}
		
		currentIndex = 0;
	}

	public String getSetName() {
		return setName;
	}
	
	public String getSetInfo() {
		return setInfo;
	}
	
	public int getDBNumber() {
		return dbNumber;
	}
	
	public int size() {
		return list.size();
	}
	
	public int getScore() {
		int count=0;
		
		for (int i=0 ; i<list.size() ; i++) {
			if (((Level)list.elementAt(i)).isCompleted()) {
				count++;
			}
		}
		
		return count;
	}
	
	public Level getLevel() {
		return (Level)list.elementAt(currentIndex);
	}
	
	public int getLevelIndex() {
		return currentIndex;
	}
	
	public boolean setLevel(int newCurrent) {
		if (newCurrent >= 0 && newCurrent < list.size()) {
			currentIndex = newCurrent;
			return true;
		}
		
		return false;
	}
	
	public boolean nextUnchallengedLevel() {
		if (this.size() == this.getScore()) {
			return false;
		}
		
		currentIndex++;
		if (currentIndex == list.size()) {
			currentIndex = 0;
		}
		
		while(((Level)list.elementAt(currentIndex)).isCompleted()) {
			currentIndex++;
			if (currentIndex == list.size()) {
				currentIndex = 0;
			}
		}
		
		return true;
	}
	
	public Vector getList() {
		return list;
	}
	
	public void reset() {
		currentIndex = 0;
		
		for (int i=0 ; i<list.size() ; i++) {
			((Level)list.elementAt(i)).reset();
		}
	}
}	
