/*
 *                 [[ Frozen Motion ]]
 *
 * Copyright (c) 2004 Glenn Sanson.
 *
 * This code is distributed under the GNU General Public License 
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * version 2, as published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 * Artwork:
 *    Amaury Amblard-Ladurantie <amaury at linuxfr.org>
 *      (the bubbles)
 *
 * Design & Programming:
 *    Glenn Sanson <glenn.sanson at free.fr>
 *      (whole Java sourcecode, including JIGA classes 
 *             http://glenn.sanson.free.fr/jiga/)
 *
 *          [[ http://glenn.sanson.free.fr/fmotion/home.html ]]
 */

import net.library.jiga.*;
import java.awt.*;
import java.util.Vector;

public class LevelSelector extends GameScreen {
	
	public final static int KEY_UP = 38;
	public final static int KEY_DOWN = 40;	
	public final static int KEY_LEFT = 37;
	public final static int KEY_RIGHT = 39;
	public final static int KEY_SHIFT = 16;
	public final static int KEY_RETURN = 10;
	public final static int KEY_ESCAPE = 27;
	
	private Image background;
	
	private Image levelBack;
	private Image miniBlocks;
	private Image mark;
	
	private LevelManager levelManager;
	private SmallFont smallFont;
	
	private ImageSprite selection;
	private int selectedLevel;
	
	private int lastKeyPressed;
	
	private boolean activate;
	
    public LevelSelector(GameApplet gameApplet) {
        super(gameApplet);
		
		GameMedia media = gameApplet.getGameMedia();
		background = media.loadImage("background.gif");

		// Level manager
		levelManager = (LevelManager)gameApplet.getGameContext().getObject("levelManager");
		
		// Font
		BigFont bigFont = (BigFont)gameApplet.getGameContext().getObject("bigFont");
		smallFont = (SmallFont)gameApplet.getGameContext().getObject("smallFont");
		
		// Init small level picture
		levelBack = media.loadImage("map-default.gif");
		miniBlocks = media.loadImage("5-blocks.gif");
		mark = media.loadImage("mark.gif");
		
		int size = this.getTextSize(levelManager.getSetName(), bigFont);
		this.addSprite(new TextSprite(new Rectangle((640 - size) >> 1, 3, size, 30), bigFont, levelManager.getSetName()));
		
		this.addSprite(new ScrollingTextSprite(new Rectangle(0, 450, 640, 30), bigFont, levelManager.getSetInfo(), 1));
		
		selection = new ImageSprite(new Rectangle(18, 40, 108, 84), media.loadImage("map-selected.gif"));
		this.addSprite(selection);
		
		selectedLevel = levelManager.getLevelIndex();
		moveSelection();
	}	
	
    public void initBackground() {
		addToBackground(background, new Point(0, 0));
		
		// Init levels
		Vector levelsList = levelManager.getList();
		int position = 0;
		
		for (int j=0 ; j<4 ; j++) {
			for (int i=0 ; i<5 ; i++) {
				if (position == levelsList.size()) {
					return;
				}
				
				// back
				addToBackground(levelBack, new Point(18+i*124, 40+j*100));
				
				// blocks
				Level currentLevel = (Level)levelsList.elementAt(position);
				
				byte[][] blocks = currentLevel.getWalls();
				
				for (int y=0 ; y<12 ; y++) {
					for (int x=0 ; x<16 ; x++) {
						if (blocks[x][y] != 0) {
							addToBackground(miniBlocks, 
											new Point(30+i*124+x*6-blocks[x][y]*6, 46+j*100+y*6),
											new Rectangle(24+i*124+x*6, 46+j*100+y*6, 6, 6));
						}
					}
				}
				
				// Is level Finished?
				if (currentLevel.isCompleted()) {
					addToBackground(mark, new Point(47+i*124, 50+j*100));
				}
				
				// name (All required operations already exist at Sprite level -> no background objects)
				int textSize = getTextSize(currentLevel.getName(), smallFont);
				this.addSprite(new TextSprite(new Rectangle(72+i*124-textSize/2, 125+j*100, textSize, 8), smallFont, currentLevel.getName()));
				
				// Difficulty
				String difficulty = new String();
				for (int x=0 ; x<currentLevel.getDifficulty() ; x++) {
					difficulty += "";
				}
				textSize = getTextSize(difficulty, smallFont);
				this.addSprite(new TextSprite(new Rectangle(72+i*124-textSize/2, 133+j*100, textSize, 8), smallFont, difficulty));				
				
				position++;
			}
		}
    }
	
	
	public int getTextSize(String text, GameFont font) {
		int size = 0;
		
		for (int i=0 ; i<text.length() ; i++) {
			size += font.charSize(text.charAt(i));
			size += font.SEPARATOR_WIDTH;
		}
		
		return size - font.SEPARATOR_WIDTH;
	}
	
	void moveSelection() {
		selection.absoluteMove(new Point(18+(selectedLevel%5)*124, 40+((int)(selectedLevel/5))*100));
	}
			
    public void play(int[] keyCodes, Point mouseCoord, boolean leftButton, boolean rightButton) {
		
		if (keyCodes.length == 0) {
			activate = true;
		}
		
		if (!activate) {
			return;
		}
		
		if (keyCodes.length != 1) {
			lastKeyPressed = 0;
			return;
		}
		
		if (keyCodes[0] == lastKeyPressed) {
			return;
		}
		
		lastKeyPressed = keyCodes[0];
		
		if (keyCodes[0] == this.KEY_UP) {
			selectedLevel -= 5;
			
			if (selectedLevel < 0) {
				selectedLevel += 20;
				
				while (selectedLevel >= levelManager.getList().size()) {
					selectedLevel -= 5;
				}
			}
			
			moveSelection();
		}
		else if (keyCodes[0] == this.KEY_DOWN) {

		    selectedLevel += 5;
			
			if (selectedLevel >= levelManager.getList().size()) {
				selectedLevel -= 20;
				
				while (selectedLevel < 0) {
					selectedLevel += 5;
				}
			}
			
			moveSelection();
		}
		else if (keyCodes[0] == this.KEY_LEFT) {

			if ((selectedLevel % 5) == 0) {
				selectedLevel += 4;
			}
			else {
				selectedLevel -= 1;
			}
			
			while (selectedLevel >= levelManager.getList().size()) {
				selectedLevel--;
			}
			
			moveSelection();
		}
		else if (keyCodes[0] == this.KEY_RIGHT) {

			if ((selectedLevel % 5) == 4) {
				selectedLevel -= 4;
			}
			else {
				selectedLevel += 1;
			}
			
			
			while (selectedLevel >= levelManager.getList().size()) {
				selectedLevel -= selectedLevel % 5;
			}
			
			moveSelection();
		}
		else if (keyCodes[0] == this.KEY_SHIFT || keyCodes[0] == this.KEY_RETURN) {
		
			levelManager.setLevel(selectedLevel);
			this.getGameApplet().setCurrentScreen(new Game(this.getGameApplet()));
		}
		else if (keyCodes[0] == this.KEY_ESCAPE) {
			
			if (levelManager.getScore() == 0) {
				levelManager.reset();
				this.getGameApplet().setCurrentScreen(new SplashScreen(this.getGameApplet()));
			}
			else {
				this.getGameApplet().setCurrentScreen(new HighscoresScreen(this.getGameApplet()));
			}
		}
	}
}