/*
 *                 [[ Frozen Motion ]]
 *
 * Copyright (c) 2004 Glenn Sanson.
 *
 * This code is distributed under the GNU General Public License 
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * version 2, as published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 * Artwork:
 *    Amaury Amblard-Ladurantie <amaury at linuxfr.org>
 *      (the bubbles)
 *
 * Design & Programming:
 *    Glenn Sanson <glenn.sanson at free.fr>
 *      (whole Java sourcecode, including JIGA classes 
 *             http://glenn.sanson.free.fr/jiga/)
 *
 *          [[ http://glenn.sanson.free.fr/fmotion/home.html ]]
 */

import java.awt.Point;

public class SpeedVector {

	public double x, y;
	
	public SpeedVector() {
		this(0., 0.);
	}

	public SpeedVector(SpeedVector s) {
		this(s.getX(), s.getY());
	}	
	
	public SpeedVector(double x, double y) {
		this.x = x;
		this.y = y;		
	}
	
	public void addVector(SpeedVector s) {
		x += s.getX();
		y += s.getY();
	}

	public void removeVector(SpeedVector s) {
		x -= s.getX();
		y -= s.getY();
	}	
	
	public double getX() {
		return x;
	}
	
	public double getY() {
		return y;
	}
	
	public void resizeVector(double value) {
		x = x * value;
		y = y * value;
	}
	
	public SpeedVector addToMax(SpeedVector sv, double max) {

		if (sv.x == 0. && sv.y == 0.) {
			return this;
		}
		
		double currentSize = x * x + y * y;
		
		if (currentSize > (max * max)) {
			return this;
		}
		
		double a_value = sv.x * sv.x + sv.y * sv.y;
		double b_value = 2 * (x * sv.x + y * sv.y);
		double c_value = currentSize - max * max;
		
		// If vector is not null, delta > 0, (-b + sqrt(delta))/2 >= 0
		double value = (-b_value + Math.sqrt(b_value * b_value - 4 * a_value * c_value)) / 2.;
		
		if (value > 1.) {
			value = 1.;
		}
		
		x += value * sv.x;
		y += value * sv.y;
		
		return this;
	}
}