/*
 *                 [[ Frozen Motion ]]
 *
 * Copyright (c) 2004 Glenn Sanson.
 *
 * This code is distributed under the GNU General Public License 
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * version 2, as published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 * Artwork:
 *    Amaury Amblard-Ladurantie <amaury at linuxfr.org>
 *      (the bubbles)
 *
 * Design & Programming:
 *    Glenn Sanson <glenn.sanson at free.fr>
 *      (whole Java sourcecode, including JIGA classes 
 *             http://glenn.sanson.free.fr/jiga/)
 *
 *          [[ http://glenn.sanson.free.fr/fmotion/home.html ]]
 */

import net.library.jiga.*;
import java.awt.*;

import java.util.Vector;
import java.util.Random;

public class SplashScreen extends Game {
	
	final static double MIN_ROTATE_ANGLE = Math.PI / 18.;
	final static double MAX_ACCELERATE_ANGLE = Math.PI / 6.;
	final static double MAX_ACCELERATE_ANGLE_SD = Math.PI / 30.;
	final static double SPEED_DOWN_DISTANCE = 30.;
	
	final static String FMOTION_TITLE = "frozen motion";
	final static String FMOTION_VERSION = "version 0.1";
	final static String FMOTION_PRESS_FIRE_1 = "press fire";
	final static String FMOTION_PRESS_FIRE_2 = "to start";
	final static String FMOTION_SCROLLING_TEXT = "frozen motion v0.1  first public release  graphics +bubbles+ by amaury amblard ladurantie +amaury@linuxfr.org+  game design +everything else+ by glenn sanson +glenn.sanson@free.fr+ ";
	
	final static int WALL_MAX_TIMER = 400;
	final static int EMPTY_MAX_TIMER = 100;
	final static int PRESS_FIRE_MAX_TIMER = 30;
	
	final static int BLOC_VERTICAL = 0;
	final static int BLOC_HORIZONTAL = 1;
	
	final static int NUMBER_OF_WALLS = 10;
	
	BubbleSprite target;
	BubbleSprite targetGoal;
	int targetType;
	
	int wallTimer;
	int emptyTimer;
	int pressFireTimer;
	boolean pressFireHidden;
	
	Rectangle[] currentWalls;
	
	TextSprite title, version;
	TextSprite pressFire, toStart;
	ScrollingTextSprite scrollingText;
	
    public SplashScreen(GameApplet gameApplet) {
		super(gameApplet);
		
		// Change parameters
		parameters = Level.DEFAULT_PARAMETERS;
		
		// Remove useless elements
		// Useless bubbles
		for (int i=0 ; i<staticBubbles.size() ; i++) {
			this.removeSprite((Sprite)staticBubbles.elementAt(i));
		}
		staticBubbles.removeAllElements();

		// Add cursor
		player = new CursorSprite(new Rectangle((randExplosion.nextInt()%300)+300, (randExplosion.nextInt()%220)+220, 32, 32), playerImage, this);
		this.addStaticSprite(player);		
		
		
		// Useless elastics
		for (int i=0 ; i<elastics.size() ; i++) {
			this.removeSprite((Sprite)elastics.elementAt(i));
		}
		elastics.removeAllElements();
		
		// Remove Walls
		wallsModel = new byte[16][12];
		currentWalls = new Rectangle[NUMBER_OF_WALLS];
						
		// Remove LifeManager sprite
		// LifeManager MUSTN'T be null
		this.removeSprite(lifeManager);
		
		// Remove Description
		for (int i=0 ; i<textVector.size() ; i++) {
			this.removeSprite((Sprite)textVector.elementAt(i));
		}
		textVector.removeAllElements();
		
		initBubbles();
		initWalls();
		
		// Init title
		int size = this.getTextSize(FMOTION_TITLE, bigFont);
		title = new TextSprite(new Rectangle((gameApplet.getWidth() - size) >> 1, 3, size, 30), bigFont, FMOTION_TITLE);
		
		this.addSprite(title);
				
		size = this.getTextSize(FMOTION_VERSION, smallFont);
		version = new TextSprite(new Rectangle((gameApplet.getWidth() - size) >> 1, 30, size, 30), smallFont, FMOTION_VERSION);
		
		this.addSprite(version);		
		
		// Init press fire
		size = this.getTextSize(FMOTION_PRESS_FIRE_1, bigFont);
		pressFire = new TextSprite(new Rectangle((gameApplet.getWidth() - size) >> 1, 210, size, 30), bigFont, FMOTION_PRESS_FIRE_1);
		
		this.addSprite(pressFire);

		size = this.getTextSize(FMOTION_PRESS_FIRE_2, bigFont);
		toStart = new TextSprite(new Rectangle((gameApplet.getWidth() - size) >> 1, 240, size, 30), bigFont, FMOTION_PRESS_FIRE_2);
		
		this.addSprite(toStart);
		
		// Init scrolling text
		scrollingText = new ScrollingTextSprite(new Rectangle(0, 450, 640, 30), bigFont, this.FMOTION_SCROLLING_TEXT, 1);
			
		this.addSprite(scrollingText);		
	}
	
	// create random bubbles
	void initBubbles() {
		
		int value = 0;
		
		while (value<6) {
			Rectangle rect = new Rectangle(Math.abs(randExplosion.nextInt()%608), Math.abs(randExplosion.nextInt()%448), 32, 32);

			boolean collision = false;
			
			if (player.getRealPosition().intersects(rect)) {
				collision = true;
			}
			
			for (int i=0 ; i<currentWalls.length ; i++) {
				if (currentWalls[i] != null) {
					if (currentWalls[i].intersects(rect)) {
						collision = true;
					}
				}
			}
			
			for (int i=0 ; i<staticBubbles.size() ; i++) {
				if (((BubbleSprite)staticBubbles.elementAt(i)).getRealPosition().intersects(rect)) {
					collision = true;
				}				
			}
			
			if (!collision) {
				this.addStaticSprite(new BubbleSprite(rect, bubbles[value%3], BubbleSprite.TYPE_NORMAL,this));
				value++;
			}
		}
		
		// Add Elastics
		int nbElastics = Math.abs(randExplosion.nextInt()%3);
		int elasticPosition = -1;
		
		for (int i=0 ; i<nbElastics ; i++) {
		
			int newPosition = Math.abs(randExplosion.nextInt() % 6);
			while (newPosition == elasticPosition) {
				newPosition = Math.abs(randExplosion.nextInt() % 6);
			}
			
			elasticPosition = newPosition;
			
			BubbleSprite extend = player;
			if (elasticPosition+1 < staticBubbles.size()) {
				extend = (BubbleSprite)staticBubbles.elementAt(elasticPosition+1);
			}
			
			Elastic e = new Elastic((BubbleSprite)staticBubbles.elementAt(elasticPosition), extend, 
									new Rectangle(0, 0, this.getGameApplet().getWidth(), this.getGameApplet().getHeight()));
			elastics.addElement(e);
			this.spriteToBack(e);			
		}			
	}
	
	// change walls
	void initWalls() {
		wallsModel = new byte[16][12];
		
		byte value = 0;
		
		while (value<NUMBER_OF_WALLS) {
			int posX = Math.abs(randExplosion.nextInt()%15);
			int posY = Math.abs(randExplosion.nextInt()%11);
			int direction = Math.abs(randExplosion.nextInt()%2);
			
			Rectangle rect = null;
			if (direction == this.BLOC_HORIZONTAL) {
				rect = new Rectangle(posX*40, posY*40, 80, 40);
			}
			else {
				rect = new Rectangle(posX*40, posY*40, 40, 80);
			}
			
			boolean collision = false;
			
			for (int i=0 ; i<staticBubbles.size() ; i++) {
				if (((BubbleSprite)staticBubbles.elementAt(i)).getRealPosition().intersects(rect)) {
					collision = true;
				}
			}
			for (int i=0 ; i<movingBubbles.size() ; i++) {
				if (((BubbleSprite)movingBubbles.elementAt(i)).getRealPosition().intersects(rect)) {
					collision = true;
				}
			}
			
			if (!collision) {
				currentWalls[value] = rect;
				wallsModel[posX][posY] = (byte)(value%5+1);

				if (direction == this.BLOC_HORIZONTAL) {
					wallsModel[posX+1][posY] = (byte)(value%5+1);
				}
				else {
					wallsModel[posX][posY+1] = (byte)(value%5+1);
				}
				
				value++;
			}
		}
	}
	
	// get color number
	int getColorNumber(Image img) {
		for (int i=0 ; i<3 ; i++) {
			if (img == bubbles[i][0]) {
				return i;
			}
		}
		
		return -1; // Never fails
	}
	
	// Select a target
	// input : target = null ; targetType = -1 ;
	void selectTarget() {
		
		double minDistance = 0.;
		BubbleSprite current = null;
		
		// Search for mini
		for (int i=0 ; i<staticBubbles.size() ; i++) {
			current = (BubbleSprite)staticBubbles.elementAt(i);
			
			if (current.getType() == BubbleSprite.TYPE_MINI) {
				if (target == null) {
					target = current;
					targetType = BubbleSprite.TYPE_MINI;
					minDistance = Math.pow(current.getRealX()-player.getRealX(), 2.)+Math.pow(current.getRealY()-player.getRealY(), 2.);
				}
				else {
					double distance = Math.pow(current.getRealX()-player.getRealX(), 2.)+Math.pow(current.getRealY()-player.getRealY(), 2.);
					
					if (distance < minDistance) {
						target = current;
						minDistance = distance;
					}
				}
			}			
		}
		
		for (int i=0 ; i<movingBubbles.size() ; i++) {
			current = (BubbleSprite)movingBubbles.elementAt(i);
			
			if (current.getType() == BubbleSprite.TYPE_MINI) {
				if (target == null) {
					target = current;
					targetType = BubbleSprite.TYPE_MINI;
					minDistance = Math.pow(current.getRealX()-player.getRealX(), 2.)+Math.pow(current.getRealY()-player.getRealY(), 2.);
				}
				else {
					double distance = Math.pow(current.getRealX()-player.getRealX(), 2.)+Math.pow(current.getRealY()-player.getRealY(), 2.);
					
					if (distance < minDistance) {
						target = current;
						// targetType is already set to TYPE_MINI
						minDistance = distance;
					}
				}
			}			
		}
		
		if (target != null) {
			return;
		}
		
		// Search for real bubble
		
		// how many bubbles are there for each color?
		int[] nbBubbles = new int[3];
		for (int i=0 ; i<staticBubbles.size() ; i++) {
			current = (BubbleSprite)staticBubbles.elementAt(i);
			
			if (current.getType() == BubbleSprite.TYPE_NORMAL) {
				nbBubbles[this.getColorNumber(current.getImage())]++;
			}
		}
		for (int i=0 ; i<movingBubbles.size() ; i++) {
			current = (BubbleSprite)movingBubbles.elementAt(i);
			
			if (current.getType() == BubbleSprite.TYPE_NORMAL) {
				nbBubbles[this.getColorNumber(current.getImage())]++;
			}
		}
		
		// Check if at least one is removable
		if (nbBubbles[0] < 2 && nbBubbles[1] < 2 && nbBubbles[2] < 2) {
			return;
		}
		
		// Search for nearest removable one
		for (int i=0 ; i<staticBubbles.size() ; i++) {
			current = (BubbleSprite)staticBubbles.elementAt(i);
			
			if (current.getType() == BubbleSprite.TYPE_NORMAL && nbBubbles[this.getColorNumber(current.getImage())] > 1) {
				
				if (target == null) {				
					target = current;
					targetType = BubbleSprite.TYPE_NORMAL;
					minDistance = Math.pow(current.getRealX()-player.getRealX(), 2.)+Math.pow(current.getRealY()-player.getRealY(), 2.);
				}
				else {
					double distance = Math.pow(current.getRealX()-player.getRealX(), 2.)+Math.pow(current.getRealY()-player.getRealY(), 2.);
					
					if (distance < minDistance) {
						target = current;
						// targetType is already set to TYPE_NORMAL
						minDistance = distance;
					}
				}
			}
		}
		for (int i=0 ; i<movingBubbles.size() ; i++) {
			current = (BubbleSprite)movingBubbles.elementAt(i);
			
			if (current.getType() == BubbleSprite.TYPE_NORMAL && nbBubbles[this.getColorNumber(current.getImage())] > 1) {
				
				if (target == null) {				
					target = current;
					targetType = BubbleSprite.TYPE_NORMAL;
					minDistance = Math.pow(current.getRealX()-player.getRealX(), 2.)+Math.pow(current.getRealY()-player.getRealY(), 2.);
				}
				else {
					double distance = Math.pow(current.getRealX()-player.getRealX(), 2.)+Math.pow(current.getRealY()-player.getRealY(), 2.);
					
					if (distance < minDistance) {
						target = current;
						// targetType is already set to TYPE_NORMAL
						minDistance = distance;
					}
				}
			}
		}
		
		// Search for targetGoal
		for (int i=0 ; i<staticBubbles.size() ; i++) {
			current = (BubbleSprite)staticBubbles.elementAt(i);
			
			if (current.getType() == target.getType() && current.getImage() == target.getImage() && current != target) {
				targetGoal = current;
				return;
			}
		}
		for (int i=0 ; i<movingBubbles.size() ; i++) {
			current = (BubbleSprite)movingBubbles.elementAt(i);
			
			if (current.getType() == target.getType() && current.getImage() == target.getImage() && current != target) {
				targetGoal = current;
				return;
			}
		}
		
	}
	
    public void play(int[] keyCodes, Point mouseCoord, boolean leftButton, boolean rightButton) {
		
		for (int i=0 ; i<keyCodes.length ; i++) {
			if (keyCodes[i] == this.KEY_UP || keyCodes[i] == this.KEY_SHIFT) {
				this.getGameApplet().setCurrentScreen(new Game(this.getGameApplet()));
				return;
			}
		}
		
		if (target == null) {
			selectTarget();
		}
		
		if (target != null) {
			double dx = 0.;
			double dy = 0.;
			
			if (targetType == BubbleSprite.TYPE_MINI) {
				// The target is the ball itself
				dx = player.getRealX()-target.getRealX();
				dy = player.getRealY()-target.getRealY();
			}
			else {
				// The target is behind the ball (to push it)
				double disTargetX = target.getRealX()-targetGoal.getRealX();
				double disTargetY = target.getRealY()-targetGoal.getRealY();
				
				// Avoid border effect
				if (disTargetX > this.getGameApplet().getWidth()/2) {
					disTargetX -= this.getGameApplet().getWidth();
				}
				else if (disTargetX < -this.getGameApplet().getWidth()/2) {
					disTargetX += this.getGameApplet().getWidth();
				}
				if (disTargetY > this.getGameApplet().getHeight()/2) {
					disTargetY -= this.getGameApplet().getHeight();
				}
				else if (disTargetY < -this.getGameApplet().getHeight()/2) {
					disTargetY += this.getGameApplet().getHeight();
				}
				
				// N.B.: Normal value should be 32, 28 is less precise but a bit faster!!!
				double ratio = 28. / Math.sqrt(disTargetX * disTargetX + disTargetY * disTargetY);
				
				dx = player.getRealX()-target.getRealX()-disTargetX*ratio;
				dy = player.getRealY()-target.getRealY()-disTargetY*ratio;
			}
			
			// Avoid border effect
			if (dx > this.getGameApplet().getWidth()/2) {
				dx -= this.getGameApplet().getWidth();
			}
			else if (dx < -this.getGameApplet().getWidth()/2) {
				dx += this.getGameApplet().getWidth();
			}
			if (dy > this.getGameApplet().getHeight()/2) {
				dy -= this.getGameApplet().getHeight();
			}
			else if (dy < -this.getGameApplet().getHeight()/2) {
				dy += this.getGameApplet().getHeight();
			}
			
			
			double angle = 0.;
			
			if (dx == 0.) {
				if (dy > 0.) {
					angle = -Math.PI/2;
				} else {
					angle = Math.PI/2;
				}
			}
			else {
				angle = Math.atan(dy/dx);
				
				if (dx>0.) {
					angle += Math.PI;
				}
			}
			
			double delta = angle - player.getAngle();
			if (delta>Math.PI) {
				delta -= 2*Math.PI;
			}
			
			if (Math.abs(delta) > this.MIN_ROTATE_ANGLE) {
				player.rotate(delta<0);
			}
			
			if (Math.abs(dx) < this.SPEED_DOWN_DISTANCE && Math.abs(dy) < this.SPEED_DOWN_DISTANCE) {
				if (Math.abs(delta) < this.MAX_ACCELERATE_ANGLE_SD) {
					player.accelerate();
				}
			}
			else {
				if (Math.abs(delta) < this.MAX_ACCELERATE_ANGLE) {
					player.accelerate();
				}				
			}
			
			if (target.getType() == BubbleSprite.TYPE_REMOVED) {
				target = null;
				targetGoal = null;
				targetType = -1;
			}
			else if (targetType == BubbleSprite.TYPE_NORMAL) {
				if (targetGoal.getType() == BubbleSprite.TYPE_REMOVED) {
					target = null;
					targetGoal = null;
					targetType = -1;
				}
			}
			else { // targetType == TYPE_MINI
				if (target.getType() != BubbleSprite.TYPE_MINI) {
					target = null;
					targetGoal = null;
					targetType = -1;
				}
			}
		}
		
		// Move bubbles
		for (int i=0 ; i<movingBubbles.size() ; i++) {
			if (((BubbleSprite)movingBubbles.elementAt(i)).moveBubble()) {
				soundManager.playSound(FrozenMotion.SOUND_COLLISION);
			}
		}

		// Collision S<->M
		for (int i=0 ; i<movingBubbles.size() ; i++) {
			BubbleSprite current = (BubbleSprite)movingBubbles.elementAt(i);
			
			for (int j=0 ; j<staticBubbles.size() ; j++) {
				if (current.checkCollision((BubbleSprite)staticBubbles.elementAt(j))) {
					if (current.getType() != BubbleSprite.TYPE_REMOVED) {
						soundManager.playSound(FrozenMotion.SOUND_BUBBLES);
					}
				}
			}
		}		

		// Collision M<->M
		for (int i=0 ; i<movingBubbles.size()-1 ; i++) {
			BubbleSprite current = (BubbleSprite)movingBubbles.elementAt(i);
			
			for (int j=i+1 ; j<movingBubbles.size() ; j++) {
				if (current.checkCollision((BubbleSprite)movingBubbles.elementAt(j))) {
					if (current.getType() != BubbleSprite.TYPE_REMOVED) {
						soundManager.playSound(FrozenMotion.SOUND_BUBBLES);
					}
				}
			}
		}
	
		// Integrate new moving bubbles
		for (int i=0 ; i<staticBubbles.size() ; i++) {
			BubbleSprite current = (BubbleSprite)staticBubbles.elementAt(i);
			
			if (current.isMoving()) {
				staticBubbles.removeElement(current);
				movingBubbles.addElement(current);
			}
		}		
		
		// Reduce speed
		for (int i=0 ; i<movingBubbles.size() ; i++) {
			BubbleSprite current = (BubbleSprite)movingBubbles.elementAt(i);
			
			if (current.reduceSpeed()) {
				movingBubbles.removeElement(current);
				staticBubbles.addElement(current);
			}
		}

		// Increase time
		// Reset Collision detection
		for (int i=0 ; i<staticBubbles.size() ; i++) {
			((BubbleSprite)staticBubbles.elementAt(i)).increaseTime();
		}
		for (int i=0 ; i<movingBubbles.size() ; i++) {
			((BubbleSprite)movingBubbles.elementAt(i)).increaseTime();
		}
		
		// Move Elastics
		for (int i=0 ; i<elastics.size() ; i++) {
			((Elastic)elastics.elementAt(i)).check(this);
		}
		
		// Calculate explosions
		for (int i=0 ; i<explosions.size() ; i++) {
			((Explosion)explosions.elementAt(i)).move(this);
		}
		
		// Check pop
		for (int i=0 ; i<popImages.size() ; i++) {
			((PopImage)popImages.elementAt(i)).increaseTime(this);
		}
		
		// Change wall timer
		wallTimer++;
		
		if (wallTimer >= this.WALL_MAX_TIMER) {
			wallTimer = 0;
			
			initWalls();
			initBackground();
		}
		
		// Reset bubbles
		if (staticBubbles.size() + movingBubbles.size() == 1) {
			emptyTimer++;
			
			if (emptyTimer >= this.EMPTY_MAX_TIMER) {
				emptyTimer = 0;
				
				initBubbles();
			}
		}
		
		// Show text [to top]
		this.spriteToFront(title);
		this.spriteToFront(version);
		this.spriteToFront(scrollingText);
		
		this.removeSprite(pressFire);
		this.removeSprite(toStart);

		if (!pressFireHidden) {
			this.addSprite(pressFire);
			this.addSprite(toStart);
		}
		
		pressFireTimer++;
		
		if (pressFireTimer >= this.PRESS_FIRE_MAX_TIMER) {
			pressFireTimer = 0;
			pressFireHidden = !pressFireHidden;
		}
   }
	
}